#!/bin/bash

echo $@
echo pwd
pwd
toolpath=/galaxy-prod/galaxy/tools-dependencies
hisat2Path=${toolpath}/bin
tmapPath=${toolpath}/bin
prinseqPath=${toolpath}/bin
bedtoolsPath=${toolpath}/bin
tempDir=/galaxy-prod/tmp
isoem2Path=${toolpath}/bin/isoem2/bin


arg=($*)
i=0
for a in ${arg[*]}
do
((i++))
	if [ "$a" == "--input1" ]; then
		RNAseq_1=${arg[i]}
	fi

        if [ "$a" == "--input2" ]; then
		RNAseq_2=${arg[i]}
	fi

	if [ "$a" == "--GTF" ]; then
		GTF_file=${arg[i]}
	fi

        if [ "$a" == "--TMAP_INDEX" ]; then
		TMAP_INDEX_file=${arg[i]}
	fi

        if [ "$a" == "--HISAT2_INDEX" ]; then
		HISAT2_INDEX_file=${arg[i]}
        fi

	if [ "$a" == "--Cluster" ]; then
		Cluster_file=${arg[i]}
	fi

	if [ "$a" == "-m" ]; then
		M=${arg[i]}
	fi

	if [ "$a" == "-d" ]; then
		D=${arg[i]}
	fi

	if [ "$a" == "--out_gene_fpkm" ]; then
		out_gene_fpkm=${arg[i]}
	fi

        if [ "$a" == "--out_gene_tpm" ]; then
		out_gene_tpm=${arg[i]}
	fi

        if [ "$a" == "--out_iso_fpkm" ]; then
		out_iso_fpkm=${arg[i]}
 	fi

        if [ "$a" == "--out_iso_tpm" ]; then
		out_iso_tpm=${arg[i]}
	fi

        if [ "$a" == "--out_bootstrap" ]; then
		out_bootstrap=${arg[i]}
	fi

        if [ "$a" == "--RNA_type" ]; then
		RNAseqType=${arg[i]}
	fi

        if [ "$a" == "--fastaFile" ]; then
		FastaFile=${arg[i]}
		fasta=yes
	fi
 
	if [ "$a" == "--MinReadLength" ]; then
		MinReadLengthNum=${arg[i]}
	fi

done


#Sahar 5/20/2017 adding support for user uploaded reference transcriptome
if [ "${fasta}" == "yes" ]
then
    echo building reference index and GTF
    ln -s ${FastaFile} reference.fasta
    ${hisat2Path}/hisat2-build reference.fasta reference.fasta
    HISAT2_INDEX_file=reference.fasta
    ${isoem2Path}/fastaToGTF reference.fasta
    ls -ltrh
    GTF_file=reference.fasta.GTF
    awk '{print substr($12,2,length($12)-3)}' reference.fasta.GTF | sort | uniq | awk '{print $1 "\t" $1}' > cluster.txt
    Cluster_file=cluster.txt
    ls ${Cluster_file} -ltr
    ls ${GTF_file} -ltr
    ls ${HISAT2_INDEX_file} -ltr
fi

if [ "${RNAseqType}" == "Illumina-paired-end" ]
then
        f=$(basename ${RNAseq_1})

        file_type=`echo $f | tail -c 6`
        if [ "$file_type" == "fastq" ]; then
	   ln ${RNAseq_1}  RNAseq_1.fastq
	   ln ${RNAseq_2}  RNAseq_2.fastq
        fi

        file_type=`echo $f | tail -c 4`
        if [ "$file_type" == "bam" ]; then
           echo "Convert BAM to fastq"
           ${bedtoolsPath}/bedtools  bamtofastq -i ${RNAseq_1} -fq RNAseq_1.fastq
           ${bedtoolsPath}/bedtools  bamtofastq -i ${RNAseq_2} -fq RNAseq_2.fastq
        fi

	echo @@@@@@ Number of raw Ilumina RNA-Seq reads @@@@@@@
	python /galaxy-prod/galaxy/tools/EpiSeq_Human/EpiSeq_CP_RNA_PE/v1_hg38/calculate_stat_fastq.py RNAseq_1.fastq RNAseq_2.fastq

	echo filter the RNA fastq QC less than 20 and duplicates
	perl ${prinseqPath}/prinseq-lite.pl -fastq RNAseq_1.fastq -fastq2 RNAseq_2.fastq -out_good RNA_QC_dup -out_bad null -min_qual_mean 20  -min_len ${MinReadLengthNum}

	echo @@@@@@ Number of cleaned Ilumina RNA-Seq reads @@@@@@@
	python /galaxy-prod/galaxy/tools/EpiSeq_Human/EpiSeq_CP_RNA_PE/v1_hg38/calculate_stat_fastq.py RNA_QC_dup_1.fastq RNA_QC_dup_2.fastq

        ${hisat2Path}/hisat2 -x ${HISAT2_INDEX_file} -1  RNA_QC_dup_1.fastq -2  RNA_QC_dup_2.fastq \
           --no-discordant --no-mixed --sensitive --no-unal --no-spliced-alignment -p 16  > RNAseq_transcriptome.sam

        rm RNA*.fastq

else # [ "${RNAseqType}" == "Illumina-single-end"  || "${RNAseqType}" == "Ion-Torrent-Proton" ] Sahar 05/05/2017 hisat2 is used for Proton - exact code as Illumina single
        f=$(basename ${RNAseq_1})

        file_type=`echo $f | tail -c 6`
        if [ "$file_type" == "fastq" ]; then
	    ln ${RNAseq_1} RNAseq_1.fastq
        fi

        file_type=`echo $f | tail -c 4`
        if [ "$file_type" == "bam" ]; then
           echo "Convert BAM to fastq"
           ${bedtoolsPath}/bedtools  bamtofastq -i ${RNAseq_1} -fq RNAseq_1.fastq
        fi

	echo @@@@@@ Number of raw Ilumina RNA-Seq reads @@@@@@@
	python /galaxy-prod/galaxy/tools/EpiSeq_Human/EpiSeq_CP_RNA_PE/v1_hg38/calculate_stat_fastq.py RNAseq_1.fastq

	echo filter the RNA fastq QC less than 20 and duplicates
	perl ${prinseqPath}/prinseq-lite.pl -fastq RNAseq_1.fastq -out_good RNA_QC_dup -out_bad null -min_qual_mean 20  -min_len ${MinReadLengthNum}

	echo @@@@@@ Number of cleaned Ilumina RNA-Seq reads @@@@@@@
	python /galaxy-prod/galaxy/tools/EpiSeq_Human/EpiSeq_CP_RNA_PE/v1_hg38/calculate_stat_fastq.py RNA_QC_dup.fastq

        ${hisat2Path}/hisat2 -x ${HISAT2_INDEX_file} -U RNA_QC_dup.fastq \
           --no-discordant --no-mixed --sensitive --no-unal --no-spliced-alignment -p 16  > RNAseq_transcriptome.sam

        rm RNA*.fastq

fi

echo Sorting
LANG=C sort -T ${tempDir} -k 1,1 RNAseq_transcriptome.sam > aligned_reads_sorted.sam


if [ "${RNAseqType}" == "Illumina-paired-end" ]
then
        echo IsoEM for paired RNAseq mapped to transcriptome
        ${isoem2Path}/isoem2 -G ${GTF_file} -c ${Cluster_file} -C 95 --polyA 200 -a aligned_reads_sorted.sam

else
        echo IsoEM for single-end RNAseq mapped to transcriptome
        ${isoem2Path}/isoem2 -G ${GTF_file} -c ${Cluster_file} --polyA 200 -C 95 -m ${M} -d ${D} aligned_reads_sorted.sam
fi

echo Join estimates files with ci files

join ./aligned_reads_sorted/output/Genes/gene_fpkm_estimates ./aligned_reads_sorted/output/ConfidenceIntervals/gene_fpkm_ci >333 
awk '{print $1 "\t" $2 "\t" $3 "\t" $4}' 333 > gene_fpkm
join ./aligned_reads_sorted/output/Genes/gene_tpm_estimates ./aligned_reads_sorted/output/ConfidenceIntervals/gene_tpm_ci |awk '{print $1 "\t" $2 "\t" $3 "\t" $4}' > gene_tpm
join ./aligned_reads_sorted/output/Isoforms/iso_fpkm_estimates ./aligned_reads_sorted/output/ConfidenceIntervals/iso_fpkm_ci |awk '{print $1 "\t" $2 "\t" $3 "\t" $4}' > iso_fpkm
join ./aligned_reads_sorted/output/Isoforms/iso_tpm_estimates ./aligned_reads_sorted/output/ConfidenceIntervals/iso_tpm_ci |awk '{print $1 "\t" $2 "\t" $3 "\t" $4}' > iso_tpm

mv ./aligned_reads_sorted/bootstrap.tar.gz ${out_bootstrap}

#4. Copy output files
#############################################################
mv gene_fpkm ${out_gene_fpkm}
mv gene_tpm ${out_gene_tpm}
mv iso_fpkm ${out_iso_fpkm}
mv iso_tpm ${out_iso_tpm}

#5.Remove files
#############################################################
rm RNAseq_transcriptome.sam
rm aligned_reads_sorted.sam
rm -rf aligned_reads_sorted

echo "done"
date



